@@ -4,7 +4,6 @@
 #include "socket.h"
 #include "packets.h"
 #include "messages.h"
-#include "timers.h"
 
 #include <linux/net.h>
 #include <linux/if_vlan.h>
@@ -63,6 +62,7 @@ static inline struct dst_entry *route(struct wireguard_device *wg, struct flowi4
 			dst = ERR_PTR(PTR_ERR(rt));
 		dst = &rt->dst;
 	} else if (addr->ss_family == AF_INET6) {
+#if IS_ENABLED(CONFIG_IPV6)
 		int ret;
 		struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)addr;
 
@@ -85,6 +85,7 @@ static inline struct dst_entry *route(struct wireguard_device *wg, struct flowi4
 #endif
 		if (unlikely(ret))
 			dst = ERR_PTR(ret);
+#endif
 	}
 	return dst;
 }
@@ -123,7 +124,7 @@ static inline int send(struct net_device *dev, struct sk_buff *skb, struct dst_e
 			ret = -ENONET;
 			goto err;
 		}
-
+#if IS_ENABLED(CONFIG_IPV6)
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 5, 0)
 		return udp_tunnel6_xmit_skb(dst, sock6, skb, dev,
 					    &fl6->saddr, &fl6->daddr,
@@ -144,6 +145,9 @@ static inline int send(struct net_device *dev, struct sk_buff *skb, struct dst_e
 				     fl6->fl6_sport, fl6->fl6_dport,
 				     false);
 		return 0;
+#endif
+#else
+		goto err;
 #endif
 	}
 
@@ -251,10 +255,8 @@ int socket_send_skb_to_peer(struct wireguard_peer *peer, struct sk_buff *skb, u8
 	read_lock_bh(&peer->endpoint_lock);
 
 	ret = send(dev, skb, dst, &peer->endpoint_flow.fl4, &peer->endpoint_flow.fl6, &peer->endpoint_addr, rcu_dereference(peer->device->sock4), rcu_dereference(peer->device->sock6), dscp);
-	if (!ret) {
-		timers_any_packet_sent(peer);
+	if (!ret)
 		peer->tx_bytes += skb_len;
-	}
 
 	read_unlock_bh(&peer->endpoint_lock);
 	rcu_read_unlock();
@@ -268,8 +270,7 @@ int socket_send_buffer_to_peer(struct wireguard_peer *peer, void *buffer, size_t
 	if (!skb)
 		return -ENOMEM;
 	skb_reserve(skb, SKB_HEADER_LEN);
-	if (likely(buffer))
-		memcpy(skb_put(skb, len), buffer, len);
+	memcpy(skb_put(skb, len), buffer, len);
 	return socket_send_skb_to_peer(peer, skb, dscp);
 }
 
@@ -395,11 +396,14 @@ static inline void set_sock_opts(struct socket *sock)
 
 int socket_init(struct wireguard_device *wg)
 {
+	struct socket *new4 = NULL;
 	struct udp_port_cfg port4 = {
 		.family = AF_INET,
 		.local_ip.s_addr = htonl(INADDR_ANY),
 		.use_udp_checksums = true
 	};
+#if IS_ENABLED(CONFIG_IPV6)
+	struct socket *new6 = NULL;
 	struct udp_port_cfg port6 = {
 		.family = AF_INET6,
 		.local_ip6 = IN6ADDR_ANY_INIT,
@@ -409,6 +413,7 @@ int socket_init(struct wireguard_device *wg)
 		.ipv6_v6only = true
 #endif
 	};
+#endif
 	struct udp_tunnel_sock_cfg cfg = {
 		.sk_user_data = wg,
 		.encap_type = 1,
@@ -416,9 +421,9 @@ int socket_init(struct wireguard_device *wg)
 	};
 
 	int ret = 0;
-	struct socket *new4 = NULL, *new6 = NULL;
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
 	int old_bindv6only;
+	struct net *nobns;
 #endif
 
 	mutex_lock(&wg->socket_update_lock);
@@ -431,37 +436,50 @@ int socket_init(struct wireguard_device *wg)
 
 	if (!wg->incoming_port)
 		wg->incoming_port = generate_default_incoming_port(wg);
-	port4.local_udp_port = port6.local_udp_port = htons(wg->incoming_port);
+	port4.local_udp_port =
+#if IS_ENABLED(CONFIG_IPV6)
+		port6.local_udp_port =
+#endif
+		htons(wg->incoming_port);
 
 	ret = udp_sock_create(wg->creating_net, &port4, &new4);
 	if (ret < 0) {
-		pr_err("Could not create outgoing IPv4 socket\n");
-		goto out;
-	}
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
-	/* Since udp_port_cfg only learned of ipv6_v6only in 4.3, we do this horrible
-	 * hack here and set the sysctl variable temporarily to something that will
-	 * set the right option for us in sock_create. It's super racey! */
-	old_bindv6only = wg->creating_net->ipv6.sysctl.bindv6only;
-	wg->creating_net->ipv6.sysctl.bindv6only = 1;
-#endif
-	ret = udp_sock_create(wg->creating_net, &port6, &new6);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
-	wg->creating_net->ipv6.sysctl.bindv6only = old_bindv6only;
-#endif
-	if (ret < 0) {
-		pr_err("Could not create outgoing IPv6 socket\n");
-		udp_tunnel_sock_release(new4);
+		pr_err("Could not create IPv4 socket\n");
 		goto out;
 	}
 
 	set_sock_opts(new4);
-	set_sock_opts(new6);
 	setup_udp_tunnel_sock(wg->creating_net, new4, &cfg);
-	setup_udp_tunnel_sock(wg->creating_net, new6, &cfg);
 	rcu_assign_pointer(wg->sock4, new4->sk);
+
+#if IS_ENABLED(CONFIG_IPV6)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 2, 0)
+	nobns = &init_net;
+#else
+	nobns = wg->creating_net;
+#endif
+	/* Since udp_port_cfg only learned of ipv6_v6only in 4.3, we do this horrible
+	 * hack here and set the sysctl variable temporarily to something that will
+	 * set the right option for us in sock_create. It's super racey! */
+	old_bindv6only = nobns->ipv6.sysctl.bindv6only;
+	nobns->ipv6.sysctl.bindv6only = 1;
+#endif
+	ret = udp_sock_create(wg->creating_net, &port6, &new6);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
+	nobns->ipv6.sysctl.bindv6only = old_bindv6only;
+#endif
+	if (ret < 0) {
+		pr_err("Could not create IPv6 socket\n");
+		udp_tunnel_sock_release(new4);
+		rcu_assign_pointer(wg->sock4, NULL);
+		goto out;
+	}
+
+	set_sock_opts(new6);
+	setup_udp_tunnel_sock(wg->creating_net, new6, &cfg);
 	rcu_assign_pointer(wg->sock6, new6->sk);
+#endif
 
 out:
 	mutex_unlock(&wg->socket_update_lock);
